function [field,model_time,coord1,coord2,levels] = retrieve_field(yfile,yfield)
% Retrieve the 2D field yfield from the file yfile

% Max length of list of fields
% This must agree with the same parameter in the
% model code
n_list_max = 32;

% First determine which type of 2D slice this is:
% xy, xz, or yz
yslice = yfile(11:12);

% Convert the field name to a code to
% enable extraction
fcode = name2code(yfield);

% Load the data file
diags = load(yfile,'-ascii');
diags = diags';

% Unpack the data

% model_time
model_time = diags(1);

% Resolution and number of levels
n1   = round(diags(2));
n2p  = round(diags(3));
nlev = round(diags(4));
ntracerp = round(diags(5));
ntracerw = round(diags(6));
ptr = 7;

% If this is a xz slice or yz slice then n2 is greater for w-level fields
if yslice == 'xy'
    n2w = n2p;
else
    n2w = n2p + 1;
end

 % Grid coordinates and levels
c1_v = diags(ptr:ptr+n1-1);    % At velocity points
ptr = ptr + n1;
c1_p = diags(ptr:ptr+n1-1);    % At pressure points
ptr = ptr + n1;
c2_v = diags(ptr:ptr+n2w-1);    % At velocity points
ptr = ptr + n2w;
c2_p = diags(ptr:ptr+n2p-1);    % At pressure points
ptr = ptr + n2p;
levels = diags(ptr:ptr+nlev-1);
ptr = ptr + nlev;

% Skip over negative levels (dummy data)
junk = diags(ptr);
while junk < 0
    ptr = ptr + 1;
    junk = diags(ptr);
end

% List of fields in the data file;
% there should be at least one zero terminating
% the list. While we're here, check that the
% desired field is in the list
inlist = 0;
fld_count = 0;
ff = diags(ptr);
while ff > 0
    fld_count = fld_count + 1;
    fld_list(fld_count) = ff;
    if ff == fcode
        inlist = 1;
    end
    ptr = ptr + 1;
    ff = diags(ptr);
end
if ~inlist
    disp(['Field ',yfield,' is not in file ',yfile])
    pause
end

% Skip dummy entries in list of fields
ptr = ptr + n_list_max - fld_count;

% And skip any remaining padding zeros
ptr = 16*ceil((ptr-1)/16) + 1;

% Size of one level of one data field
% plus scaling info plus padding
field_sizep = 16*ceil(1 + n1*n2p/16);
field_sizew = 16*ceil(1 + n1*n2w/16);

% Skip over fields until we reach the required one
fld_idx = 1;
ff = fld_list(fld_idx);
while ff ~= fcode
    %disp(['Skipping field ',num2str(ff)])
    if ff == 17
        % Cloud top height has a single level
        ptr = ptr + field_sizep;
    elseif ff == 18
        % We have ntracerp tracers on nlev levels
        ptr = ptr + nlev*ntracerp*field_sizep;
    elseif ff == 19
        % We have ntracerw tracers on nlev levels
        ptr = ptr + nlev*ntracerw*field_sizew;
    elseif ff == 1 | ff == 2 | ff == 4 | ff == 5 | ff == 21
        % we have nlev levels
        ptr = ptr + nlev*field_sizep;
    else
        % we have nlev levels
        ptr = ptr + nlev*field_sizew;
    end
    fld_idx = fld_idx + 1;
    ff = fld_list(fld_idx);
end

% Now read the required field
if ff == 17
    % Cloud top height has a single level
    field = diags(ptr:ptr + 16 + n1*n2p - 1);
    datamin = field(1);
    datamax = field(2);
    safety = field(3);
    field = reshape(field(17:end),n1,n2p)*(datamax - datamin + safety) + datamin;
    ptr = ptr + field_sizep;

elseif ff == 18
    % We have ntracerp tracers on nlev levels
    for itr = 1:ntracerp
        for ilev = 1:nlev
            temp = diags(ptr:ptr + 16 + n1*n2p - 1);
            datamin = temp(1);
            datamax = temp(2);
            safety = temp(3);
            field(:,:,ilev,itr) = reshape(temp(17:end),n1,n2p)*(datamax - datamin + safety) + datamin;
            ptr = ptr + field_sizep;
        end
    end

elseif ff == 19
    % We have ntracerw tracers on nlev levels
    for itr = 1:ntracerw
        for ilev = 1:nlev
            temp = diags(ptr:ptr + 16 + n1*n2w - 1);
            datamin = temp(1);
            datamax = temp(2);
            safety = temp(3);
            field(:,:,ilev,itr) = reshape(temp(17:end),n1,n2w)*(datamax - datamin + safety) + datamin;
            ptr = ptr + field_sizew;
        end
    end

elseif ff == 1 | ff == 2 | ff == 4 | ff == 5 | ff == 21
    % We have one field on nlev levels
    for ilev = 1:nlev
        temp = diags(ptr:ptr + 16 + n1*n2p - 1);
        datamin = temp(1);
        datamax = temp(2);
        safety = temp(3);
        field(:,:,ilev) = reshape(temp(17:end),n1,n2p)*(datamax - datamin + safety) + datamin;
        ptr = ptr + field_sizep;
    end

else
    % We have one field on nlev levels
    for ilev = 1:nlev
        temp = diags(ptr:ptr + 16 + n1*n2w - 1);
        datamin = temp(1);
        datamax = temp(2);
        safety = temp(3);
        field(:,:,ilev) = reshape(temp(17:end),n1,n2w)*(datamax - datamin + safety) + datamin;
        ptr = ptr + field_sizew;
    end

end

% Finally decide which coordinate we need.
if yslice == 'xy'
    if fcode == 1
        % The field is u
        coord1 = c1_v;
        coord2 = c2_p;
    elseif fcode == 2
        % The field is v
        coord1 = c1_p;
        coord2 = c2_v;
    else
        % Any other field
        coord1 = c1_p;
        coord2 = c2_p;
    end

elseif yslice == 'xz'
    if fcode == 1
        % The field is u
        coord1 = c1_v;
        coord2 = c2_p;
    elseif (fcode == 2 | fcode == 4 | fcode == 5 | fcode == 18 | fcode == 21)
        % The field is v, p, rho, trcp, or diss
        coord1 = c1_p;
        coord2 = c2_p;
    else
        % Any other field
        coord1 = c1_p;
        coord2 = c2_v;
    end

elseif yslice == 'yz'
    if fcode == 2
        % The field is v
        coord1 = c1_v;
        coord2 = c2_p;
    elseif (fcode == 1 | fcode == 4 | fcode == 5 | fcode == 18 | fcode == 21)
        % The field is u, p, rho, trcp, or diss
        coord1 = c1_p;
        coord2 = c2_p;
    else
        % Any other field
        coord1 = c1_p;
        coord2 = c2_v;
    end

else
    disp(['yslice = ',yslice,' not recognized '])
    pause

end

end